@@ -30,7 +30,7 @@ from reportlab.lib.colors import gray, lightgrey
 from reportlab.lib.rl_accel import fp_str
 from reportlab.lib.enums import TA_LEFT, TA_CENTER, TA_RIGHT
 from reportlab.lib.styles import _baseFontName
-from reportlab.lib.utils import strTypes, rl_safe_exec
+from reportlab.lib.utils import strTypes, rl_safe_exec, annotateException
 from reportlab.lib.abag import ABag
 from reportlab.pdfbase import pdfutils
 from reportlab.pdfbase.pdfmetrics import stringWidth
@@ -41,7 +41,7 @@ __all__ = '''AnchorFlowable BalancedColumns BulletDrawer CallerMacro CondPageBre
         HRFlowable Image ImageAndFlowables KeepInFrame KeepTogether LIIndenter ListFlowable ListItem
         Macro NullDraw PTOContainer PageBreak PageBreakIfNotEmpty ParagraphAndImage Preformatted
         SetPageTopFlowables SetTopFlowables SlowPageBreak Spacer TopPadder TraceInfo UseUpSpace XBox
-        splitLine splitLines'''.split()
+        splitLine splitLines PlacedStory'''.split()
 
 class TraceInfo:
     "Holder for info about where an object originated"
@@ -250,7 +250,6 @@ def _dedenter(text,dedent=0):
 
     return lines
 
-
 SPLIT_CHARS = "[{( ,.;:/\\-"
 
 def splitLines(lines, maximum_length, split_characters, new_line_characters):
@@ -688,13 +687,11 @@ class KeepTogether(_ContainerSpace,Flowable):
 
     def __init__(self,flowables,maxHeight=None):
         if not hasattr(KeepTogether,'NullActionFlowable'):
-            #cache these on the class
+            #store on the class; importing into the class is cirular importing
             from reportlab.platypus.doctemplate import NullActionFlowable
             from reportlab.platypus.doctemplate import FrameBreak
-            from reportlab.lib.utils import annotateException
             KeepTogether.NullActionFlowable = NullActionFlowable
             KeepTogether.FrameBreak = FrameBreak
-            self.annotateException = annotateException
 
         if not flowables:
             flowables = [self.NullActionFlowable()]
@@ -713,7 +710,7 @@ class KeepTogether(_ContainerSpace,Flowable):
         try:
             W,H = _listWrapOn(self._content,aW,self.canv,dims=dims)
         except:
-            self.annotateException('\nraised by class %s(%s)@0x%8.8x wrap\n' % (self.__class__.__name__,self.__class__.__module__,id(self)))
+            annotateException('\nraised by class %s(%s)@0x%8.8x wrap\n' % (self.__class__.__name__,self.__class__.__module__,id(self)))
         self._H = H
         self._H0 = dims and dims[0][1] or 0
         self._wrapInfo = aW,aH
@@ -1224,6 +1221,64 @@ class KeepInFrame(_Container,Flowable):
         _Container.drawOn(self, canv, x, y, _sW=_sW, scale=scale)
         if ss: canv.restoreState()
 
+class PlacedStory(Flowable):
+    _ZEROSIZE=1
+    _SPACETRANSFER = True
+    _anchors = ('nw','n','ne','w','c','e','sw','s','se')
+    def __init__(self, x, y, maxWidth, maxHeight, content=[], mergeSpace=1, mode='shrink',
+                        name='', anchor='sw', fakeWidth=None, hAlign='LEFT',vAlign='BOTTOM',
+                        showBoundary=None, origin='page'):
+        self.kif = KeepInFrame(maxWidth, maxHeight, content=content, mergeSpace=mergeSpace,
+                         mode=mode, name=name,hAlign=hAlign,vAlign=vAlign, fakeWidth=None)
+        self.x = x
+        self.y = y
+        self.anchor = anchor
+        self.w = None
+        self.h = None
+        self.sb = showBoundary
+        self.origin = origin
+
+    def wrap(self,_aW,_aH):
+        #_aW & _aH are ignored
+        if self.w is None:
+            self.kif.canv = getattr(self,'canv')
+            self.w, self.h = self.kif.wrap(self.kif.maxWidth,self.kif.maxHeight)
+            del self.kif.canv
+        return 0,0
+
+    def drawOn(self, canv, lx, ly, _sW=0):
+        #we ignore _x and _y infavour of our own
+        if self.w is None: self.wrap(0,0)
+        origin = self.origin
+        if origin=='page':
+            dx = dy = 0
+        elif origin=='local':
+            dx = lx
+            dy = ly
+        elif origin=='frame':
+            _ = getattr(self,'_frame',None)
+            if not _: _ = self._doctemplateAttr('frame')
+            dx = _.x1
+            dy = _.y1
+        else:
+            raise ValueError(f'PlacedStory invalid {origin=} not in "page", "frame" or "local"')
+
+        anchor = self.anchor
+        if anchor not in self._anchors:
+            raise ValueError(f'PlacedStory invalid {anchor=} not in {self._anchors}')
+        if anchor in ('nw','n','ne'):
+            dy -= self.h
+        elif anchor in ('w','c','e'):
+            dy -= self.h / 2
+        if anchor in ('n','c','s'):
+            dx -= self.w/2
+        elif anchor in ('ne','e','se'):
+            dx -= self.w
+        x = self.x + dx
+        y = self.y + dy
+
+        self.kif.drawOn(canv,x,y)
+        if self.sb: canv.drawBoundary(self.sb,x,y,self.w,self.h)
 
 class _FindSplitterMixin:
     def _findSplit(self,canv,availWidth,availHeight,mergeSpace=1,obj=None,content=None,paraFix=True):
@@ -1658,7 +1713,6 @@ class BalancedColumns(_FindSplitterMixin,NullDraw):
                 overlapAttachedSpace=frame._oASpace,
                 _debug=frame._debug) for i in range(nCols)]
 
-
         #we are going to modify the current template
         T=self._doctemplateAttr('pageTemplate')
         if T is None:
@@ -1944,7 +1998,6 @@ class FrameSplitter(NullDraw):
         frame.add_generated_content(*G)
         return 0,0
 
-
 from reportlab.lib.sequencer import _type2formatter
 _bulletNames = dict(
                 bulletchar=u'\u2022',   #usually a small circle
@@ -2097,21 +2150,28 @@ class LIIndenter(DDIndenter):
             self.spaceAfter = spaceAfter
 
     def split(self, aW, aH):
-        S = self._flowable.split(aW-self._leftIndent-self._rightIndent, aH)
+        f = self._flowable
+        S = f.split(aW-self._leftIndent-self._rightIndent, aH)
+        if len(S)>1 and hasattr(f,'blPara'):
+            cnl = len(f.blPara.lines)
+            nnl = len(S[0].blPara.lines)
+            #avoid 1/2 line widow when justified else avoid 1 line widow
+            if ((getattr(getattr(f,'style',None),'alignment',None)==4 and nnl>=(cnl-2)) 
+                    or nnl==(cnl-1)):
+                S = []
         return [
                 LIIndenter(s,
                         leftIndent=self._leftIndent,
                         rightIndent=self._rightIndent,
                         bullet = (s is S[0] and self._bullet or None),
                         ) for s in S
-                ]
+                ] if S else []
 
     def drawOn(self, canv, x, y, _sW=0):
         if self._bullet:
             self._bullet.drawOn(self,canv,x,y,0)
         self._flowable.drawOn(canv,x+self._leftIndent,y,max(0,_sW-self._leftIndent-self._rightIndent))
 
-
 from reportlab.lib.styles import ListStyle
 class ListItem:
     def __init__(self,
